/*
 *	getopt.c - the real thing!
 */
/*
 * Here's something you've all been waiting for:  the AT&T public domain
 * source for getopt(3c).  It is the code which was given out at the 1985
 * UNIFORUM conference in Dallas.  I [?who? -gaw] obtained it by
 * electronic mail directly from AT&T.  The people there assure me that
 * it is indeed in the public domain.
 */
/*
 * The manual page supplied with this version (i.e. smail-2.7) is from C
 * News, and was originally written by Henry Spencer for his Public
 * Domain version of getopt().
 */
/*
 * Several features have been added to the original version.
 *
 * One minor feature has been added that allows the possibility of
 * detecting some missing option arguments by looking for a leading
 * minus sign on the option argument.  This feature is enabled by
 * defining NO_MINUS at compile time.
 *
 * Normally stdio(3s) is avoided, since it can significantly increase the
 * size of a programme.  If stdio is being used anyway, getopt(3c) can be
 * compiled with USE_STDIO.
 *
 * To prevent getopt() from emitting any error messages, define a nil
 * manifest ERR; eg. "cc -DSYSVR3 -DERR -c getopt.c".
 *
 * In addition, an implementation of the getopt(1) command can be built
 * by defining GETOPT at compile time.
 */

#define SID	"@(#)smail-2:getopt.c	1.1	92/04/20 00:28:11 (woods)"
#include "sccsid.h"

#ifndef lint		/* supposedly from this version: */
static char	*orig_sccsid = "@(#)libc-port:gen/getopt.c\t1.9\t(AT&T)";
#endif

#if defined(SYSVR2) || defined(SYSVR3) || defined(SYSVR4)
# ifndef SYSV
#  define SYSV
# endif
#endif

#if !defined(SYSV) && !defined(__STDC__) && !defined(_POSIX_SOURCE)
# define strchr		index
# define strrchr	rindex
#endif

#ifndef NULL
# define NULL	0
#endif
#ifndef EOF
# define EOF	(-1)
#endif
#ifndef STDERR
# define STDERR	2
#endif

#ifdef GETOPT
# ifndef USE_STDIO
#  define USE_STDIO	1
# endif
#endif

#ifndef ERR
# ifndef USE_STDIO
#  define ERR(msg, ch)	if (opterr) { \
		static char	errbuf[3] = "?\n"; \
		errbuf[0] = ch; \
		(void) write(STDERR, argv[0], strlen(argv[0])); \
		(void) write(STDERR, msg, strlen(msg)); \
		(void) write(STDERR, errbuf, 2); \
	}
# else
#  include <stdio.h>
#  define ERR(msg, ch)	if (opterr) { \
		(void) fputs(argv[0], stderr); \
		(void) fputs(msg, stderr); \
		(void) fputc(ch, stderr); \
		(void) fputc('\n', stderr); \
	}
# endif
#else
# undef ERR
# define ERR(msg, ch)	{;}		/* we must be VERY careful with ERR() */
#endif

/* these should be safe, even on broken ANSI-C like compilers... */
extern int	strcmp();
extern char	*strchr();
extern char	*strrchr();

/*LINTLIBRARY*/

int	opterr = 1;
int	optind = 1;
int	optopt = 0;
char	*optarg = 0;

int
getopt(argc, argv, opts)
	int	argc;
	char	*argv[];
	char	*opts;
{
	static int	sp = 1;
	register int	c;
	register char	*cp;

	if (sp == 1) {
		if (optind >= argc || argv[optind][0] != '-' || argv[optind][1] == '\0')
			return(EOF);
		else if (strcmp(argv[optind], "--") == NULL) {
			optind++;
			return(EOF);
		}
	}
	optopt = c = argv[optind][sp];
	if (c == ':' || (cp = strchr(opts, c)) == NULL) {
		ERR(": illegal option -- ", c);
		if (argv[optind][++sp] == '\0') {
			optind++;
			sp = 1;
		}
		return('?');
	}
	if (*++cp == ':') {
		if (argv[optind][sp+1] != '\0')
			optarg = &argv[optind++][sp+1];
		else if (++optind >= argc) {
			ERR(": option requires an argument -- ", c);
			sp = 1;
			return('?');
#ifndef NO_MINUS
		} else
			optarg = argv[optind++];
#else /* NO_MINUS */
		} else {
			optarg = argv[optind++];
			if (*optarg == '-') {
				ERR(": option requires an argument -- ", c);
				sp = 1;
				return('?');
			}
		}
#endif /* NO_MINUS */
		sp = 1;
	} else {
		if (argv[optind][++sp] == '\0') {
			sp = 1;
			optind++;
		}
		optarg = NULL;
	}
	return(c);
}



#ifdef GETOPT

/*
 * borrowed from TransFig's fig2dev
 */
#ifndef lint
static	char	sccspid[] = "@(#) getopt.c 5.1 (WangInst) 6/15/85";
#endif

#define	ARGCH    (int)':'
#define BADCH	 (int)'?'
#define	ENDARGS  "--"

int
main(argc, argv) 
	int	argc
	char	*argv[];
{
	char	*optstring = argv[1];
	char	*argv0;
	int	opterr = 0;
	int	C;
	char	*opi;

	extern int	optind;
	extern char	*optarg;
	extern char	*strrchr();
	extern char	*strchr();

	argv0 = (argv0 = strrchr(argv[0], '/')) ? argv0 + 1 : &argv[0];
	if (argc == 1) {
		(void) fprintf(stderr, "Usage: %s optstring args ...\n", argv0);
		exit(2);
	}
	argv++;
	argc--;
	argv[0] = argv0;
	while ((C = getopt(argc, argv, optstring)) != EOF) {
		if (C == BADCH) 
			opterr++;
		(void) printf("-%c ", C);
		opi = strchr(optstring, C);
		if (opi && opi[1] == ARGCH)
			if (optarg)
				(void) printf("\"%s\" ", optarg);
			else 
				opterr++;
	}
	(void) printf("%s", ENDARGS);
	while (optind < argc)
		printf(" \"%s\"", argv[optind++]);
	putchar('\n');
	exit(opterr);
}

#endif /* GETOPT */
